@@ -1,3 +1,4 @@
+
 /*
  * Gnome Robots II
  * written by Mark Rae <m.rae@inpharmatica.co.uk>
@@ -21,6 +22,8 @@
 
 #include <config.h>
 
+#include "gnome-robots.h"
+
 #include <sys/time.h>
 #include <string.h>
 #include <stdlib.h>
@@ -28,6 +31,7 @@
 
 #include <glib/gi18n.h>
 #include <glib.h>
+#include <libgames-support.h>
 
 #include "gbdefs.h"
 #include "gameconfig.h"
@@ -36,9 +40,6 @@
 #include "properties.h"
 #include "game.h"
 #include "cursors.h"
-#include "games-gridframe.h"
-#include "games-scores.h"
-#include "games-scores-dialog.h"
 
 /* Minimum sizes. */
 #define MINIMUM_TILE_WIDTH   8
@@ -46,6 +47,16 @@
 
 #define KEY_GEOMETRY_GROUP "geometry"
 
+/**********************************************************************/
+/* Utility structs                                                    */
+/**********************************************************************/
+typedef struct
+{
+    gchar* key;
+    gchar* name;
+} key_value;
+/**********************************************************************/
+
 /**********************************************************************/
 /* Exported Variables                                                 */
 /**********************************************************************/
@@ -53,7 +64,7 @@ GtkWidget *window = NULL;
 static gint window_width = 0, window_height = 0;
 static gboolean window_is_maximized = FALSE;
 GtkWidget *game_area = NULL;
-GamesScores *highscores;
+GamesScoresContext *highscores;
 GSettings *settings;
 /**********************************************************************/
 
@@ -91,23 +102,24 @@ static const GActionEntry win_entries[] = {
   { "wait", wait_cb, NULL, NULL, NULL },
 };
 
-static const GamesScoresCategory scorecats[] = {
-  {"classic_robots", N_("Classic robots")},
-  {"classic_robots-safe", N_("Classic robots with safe moves")},
-  {"classic_robots-super-safe", N_("Classic robots with super-safe moves")},
-  {"nightmare", N_("Nightmare")},
-  {"nightmare-safe", N_("Nightmare with safe moves")},
-  {"nightmare-super-safe", N_("Nightmare with super-safe moves")},
-  {"robots2", N_("Robots2")},
-  {"robots2-safe", N_("Robots2 with safe moves")},
-  {"robots2-super-safe", N_("Robots2 with super-safe moves")},
-  {"robots2_easy", N_("Robots2 easy")},
-  {"robots2_easy-safe", N_("Robots2 easy with safe moves")},
-  {"robots2_easy-super-safe", N_("Robots2 easy with super-safe moves")},
-  {"robots_with_safe_teleport", N_("Robots with safe teleport")},
-  {"robots_with_safe_teleport-safe", N_("Robots with safe teleport with safe moves")},
-  {"robots_with_safe_teleport-super-safe", N_("Robots with safe teleport with super-safe moves")}
+static const key_value scorecats[] = {
+	{"classic_robots", N_("Classic robots")},
+	{"classic_robots-safe", N_("Classic robots with safe moves")},
+	{"classic_robots-super-safe", N_("Classic robots with super-safe moves")},
+	{"nightmare", N_("Nightmare")},
+	{"nightmare-safe", N_("Nightmare with safe moves")},
+	{"nightmare-super-safe", N_("Nightmare with super-safe moves")},
+	{"robots2", N_("Robots2")},
+	{"robots2-safe", N_("Robots2 with safe moves")},
+	{"robots2-super-safe", N_("Robots2 with super-safe moves")},
+	{"robots2_easy", N_("Robots2 easy")},
+	{"robots2_easy-safe", N_("Robots2 easy with safe moves")},
+	{"robots2_easy-super-safe", N_("Robots2 easy with super-safe moves")},
+	{"robots_with_safe_teleport", N_("Robots with safe teleport")},
+	{"robots_with_safe_teleport-safe", N_("Robots with safe teleport with safe moves")},
+	{"robots_with_safe_teleport-super-safe", N_("Robots with safe teleport with super-safe moves")}
 };
+static gint no_categories = 15;
 
 static gint safe_teleports = 0;
 static GtkWidget *safe_teleports_label;
@@ -161,6 +173,19 @@ update_game_status (gint score, gint current_level, gint safes)
   g_free (button_text);
 }
 
+const gchar *
+category_name_from_key (const gchar* key)
+{
+  int i;
+  for (i = 0; i < no_categories; i++)
+  {
+    if (g_strcmp0 (scorecats[i].key, key) == 0)
+      return scorecats[i].name;
+  }
+  /*Return key as is if match not found*/
+  return NULL;
+}
+
 static void
 preferences_cb (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
@@ -170,7 +195,7 @@ preferences_cb (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 static void
 scores_cb (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
-  show_scores (0, FALSE);
+  show_scores ();
 }
 
 static void
@@ -187,9 +212,9 @@ help_cb (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 static void
 about_cb (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
-  const gchar *authors[] = { "Mark Rae <m.rae@inpharmatica.co.uk>", "Rasoul M.P. Aghdam (player death sound)", NULL };
+  const gchar *authors[] = { "Mark Rae <m.rae@inpharmatica.co.uk>", NULL };
 
-  const gchar *artists[] = { "Kirstie Opstad <K.Opstad@ed.ac.uk>", NULL };
+  const gchar *artists[] = { "Kirstie Opstad <K.Opstad@ed.ac.uk>", "Rasoul M.P. Aghdam (player death sound)", NULL };
 
   const gchar *documenters[] =
     { "Aruna Sankaranarayanan", NULL };
@@ -197,8 +222,8 @@ about_cb (GSimpleAction *action, GVariant *parameter, gpointer user_data)
   gtk_show_about_dialog (GTK_WINDOW (window),
 			 "name", _("Robots"),
 			 "version", VERSION,
-			 "copyright", "Copyright © 1998–2008 Mark Rae\nCopyright © 2014 Michael Catanzaro",
-			 "license-type", GTK_LICENSE_GPL_2_0,
+			 "copyright", "Copyright © 1998–2008 Mark Rae\nCopyright © 2014–2016 Michael Catanzaro",
+			 "license-type", GTK_LICENSE_GPL_3_0,
 			 "comments", _("Based on classic BSD Robots"),
 			 "authors", authors,
 			 "artists", artists,
@@ -260,10 +285,7 @@ static gboolean
 window_configure_event_cb (GtkWidget *widget, GdkEventConfigure *event)
 {
   if (!window_is_maximized)
-  {
-    window_width = event->width;
-    window_height = event->height;
-  }
+    gtk_window_get_size (GTK_WINDOW (window), &window_width, &window_height);
 
   return FALSE;
 }
@@ -287,19 +309,11 @@ startup (GtkApplication *app, gpointer user_data)
 {
   struct timeval tv;
 
-  games_scores_startup ();
-
   gettimeofday (&tv, NULL);
   srand (tv.tv_usec);
 
   g_set_application_name (_("Robots"));
 
-  highscores = games_scores_new ("gnome-robots",
-                                 scorecats, G_N_ELEMENTS (scorecats),
-                                 NULL, NULL,
-                                 0 /* default category */,
-                                 GAMES_SCORES_STYLE_PLAIN_DESCENDING);
-
   settings = g_settings_new ("org.gnome.robots");
 
   gtk_window_set_default_icon_name ("gnome-robots");
@@ -313,12 +327,22 @@ shutdown (GtkApplication *app, gpointer user_data)
   g_settings_set_boolean (settings, "window-is-maximized", window_is_maximized);
 }
 
+static GamesScoresCategory *
+create_category_from_key (const char *key, gpointer user_data)
+{
+  const gchar *name = category_name_from_key (key);
+  if (name == NULL)
+    return NULL;
+  return games_scores_category_new (key, name);
+}
+
 static void
 activate (GtkApplication *app, gpointer user_data)
 {
   GtkWidget *errordialog, *vbox, *hbox, *label, *button, *gridframe;
   GtkSizeGroup *size_group;
   GtkStyleContext *style_context;
+  GamesScoresDirectoryImporter *importer;
 
   headerbar = gtk_header_bar_new ();
   gtk_header_bar_set_title (GTK_HEADER_BAR (headerbar), _("Robots"));
@@ -338,7 +362,7 @@ activate (GtkApplication *app, gpointer user_data)
   make_cursors ();
 
   game_area = gtk_drawing_area_new ();
-  gtk_widget_add_events (game_area, GDK_BUTTON_PRESS_MASK |
+  gtk_widget_add_events (game_area, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
 			 GDK_POINTER_MOTION_MASK);
   g_signal_connect (G_OBJECT (game_area), "button-press-event",
 		    G_CALLBACK (mouse_cb), NULL);
@@ -348,8 +372,10 @@ activate (GtkApplication *app, gpointer user_data)
 		    G_CALLBACK (resize_cb), NULL);
   g_signal_connect (G_OBJECT (game_area), "draw",
 		    G_CALLBACK (draw_cb), NULL);
+  g_signal_connect (G_OBJECT (game_area), "destroy",
+                    G_CALLBACK (gtk_widget_destroyed), &game_area);
 
-  gridframe = games_grid_frame_new (GAME_WIDTH, GAME_HEIGHT);
+  gridframe = GTK_WIDGET (games_grid_frame_new (GAME_WIDTH, GAME_HEIGHT));
   gtk_container_add (GTK_CONTAINER (gridframe), game_area);
 
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
@@ -398,6 +424,17 @@ activate (GtkApplication *app, gpointer user_data)
 			       MINIMUM_TILE_WIDTH * GAME_WIDTH,
 			       MINIMUM_TILE_HEIGHT * GAME_HEIGHT);
 
+  importer = games_scores_directory_importer_new ();
+  highscores = games_scores_context_new_with_importer ("gnome-robots",
+                                                       /* Label on the scores dialog, next to map type dropdown */
+                                                       _("Game Type:"),
+                                                       GTK_WINDOW (window),
+                                                       create_category_from_key,
+                                                       NULL,
+                                                       GAMES_SCORES_STYLE_POINTS_GREATER_IS_BETTER,
+                                                       GAMES_SCORES_IMPORTER (importer));
+  g_object_unref (importer);
+
   gtk_widget_show_all (window);
 
   if (!load_game_configs ()) {
@@ -409,6 +446,7 @@ activate (GtkApplication *app, gpointer user_data)
 					  _
 					  ("The program Robots was unable to find any valid game configuration files. Please check that the program is installed correctly."));
     gtk_window_set_resizable (GTK_WINDOW (errordialog), FALSE);
+
     gtk_dialog_run (GTK_DIALOG (errordialog));
     exit (1);
   }
